@echo off
call :set_errorlevel_1
setlocal ENABLEEXTENSIONS
if errorlevel 1 goto :noextensions

set TCUSERVER=1.07

rem +--------------------------------------------------------------------------+
rem | Customization                                                            |
rem +--------------------------------------------------------------------------+

rem ..................................
rem . Settings for all user accounts .
rem ..................................

rem Path for user profiles (Documents and Settings, if empty)
set TCDAS=

rem Path for TrueCrypt container files (Documents and Settings, if empty)
set TCFOLDER=

rem Extension for TrueCrypt container files
set TCEXT=tc

rem Drive letter for TrueCrypt volume
set TCDRIVE=U

rem ........................................
rem . Additional settings if TCUSER.CMD is .
rem . used for only one user account       .                                                             
rem ........................................

rem User name (%ARG2%)
set TCUSER=

rem Name of TrueCrypt volume (%TCFOLDER%\%USERNAME%.%TCEXT%, if empty)
set TCVOL=

rem ....................
rem . Further settings .
rem ....................

rem Additional TrueCrypt switches
set TCOPT=/a /q

rem Path for SUperior SU
set TCSU=%ProgramFiles%\Superior SU\su.exe

rem Dismount on logoff (if TCDOL equals 1)
set TCDOL=

rem Forced dismount on logoff (if TCFOL equals 1)
set TCFOL=

rem Dismount all TrueCrypt volumes (if TCAOL equals 1)
set TCAOL=

rem Pause on error (if TCPOE equals 1)
set TCPOE=

rem Clear screen to remove unimportant messages (if TCCLS equals 1)
set TCCLS=1

rem +--------------------------------------------------------------------------+
rem | Process settings                                                         |
rem +--------------------------------------------------------------------------+

set ProgramFiles86=%ProgramFiles(x86)%
if "%ProgramFiles86%" neq "" goto :program_files_updated
set ProgramFiles86=%ProgramFiles%
:program_files_updated

if "%TCDAS%" neq "" goto :das_updated

call :extract_path "%USERPROFILE%"
set TCDAS=%STR%
if "%TCDAS%" equ "" goto :das_missing
:das_updated
if "%TCFOLDER%" equ "" set TCFOLDER=%TCDAS%

if "%TCVOL%" equ "" goto :volume_is_valid
if "%TCUSER%" equ "" goto :username_missing
:volume_is_valid

set TCSINGLEUSER=
if "%TCUSER%" equ "" goto :no_single_user
set TCSINGLEUSER=1
:no_single_user

rem +--------------------------------------------------------------------------+
rem | Check syntax                                                             |
rem +--------------------------------------------------------------------------+

set ARG1=%~1
set ARG2=%~2
set ARG3=%~3
set ARG4=%~4
set ARG5=%~5
set ARG6=%~6
set ARG7=%~7

:process_switches
if "%ARG1%" equ "" goto :noargs
if "%ARG1:~0,1%" neq "/" goto :process_command

if /i "%ARG1%" equ "/A" goto :switch_a
if /i "%ARG1%" equ "/D" goto :switch_d
if /i "%ARG1%" equ "/F" goto :switch_f
if /i "%ARG1%" equ "/P" goto :switch_p
goto :invalid_switch

:switch_a
set TCAOL=1
goto :process_next_switch

:switch_d
set TCDOL=1
goto :process_next_switch

:switch_f
set TCFOL=1
goto :process_next_switch

:switch_p
set TCPOE=1
goto :process_next_switch

:process_next_switch
set ARG1=%ARG2%
set ARG2=%ARG3%
set ARG3=%ARG4%
set ARG4=%ARG5%
set ARG5=%ARG6%
set ARG6=%ARG7%
goto :process_switches

:process_command
set TCCMD=%ARG1%
if "%TCSINGLEUSER%" neq "" goto :tcuser_already_defined
set TCUSER=%ARG2%
set ARG2=%ARG3%
:tcuser_already_defined
if "%TCCMD%"  equ "" goto :noargs
if "%ARG2%"   neq "" goto :toomanyargs

if /i "%TCUSER%" equ "%USERNAME%" goto :invalid_username

if "%TCVOL%" neq "" goto :no_default_volume_name
set TCVOL=%TCFOLDER%\%TCUSER%.%TCEXT%
:no_default_volume_name
set TCHOME=%TCDAS%\%TCUSER%
if /i "%TCCMD%" equ "convert"     goto :convert
if /i "%TCCMD%" equ "restore"     goto :restore
if /i "%TCCMD%" equ "unmap"       goto :unmap
if /i "%TCCMD%" equ "map"         goto :map
if /i "%TCCMD%" equ "mount"       goto :mount
if /i "%TCCMD%" equ "unmount"     goto :unmount
if /i "%TCCMD%" equ "start"       goto :start
if /i "%TCCMD%" equ "su"          goto :su
if /i "%TCCMD%" equ "explore"     goto :explore
if /i "%TCCMD%" equ "replacewith" goto :replacewith
if /i "%TCCMD%" equ "register"    goto :register

:noargs
:toomanyargs
:invalid_switch
if "%TCPOE%" neq "1" goto :noargs2
call tcuser | more
goto :error

:noargs2
call :search_for_truecrypt
call :search_for_truecryptformat
call :print_title

rem ----------------------------------------------------------------------------
if "%TCSINGLEUSER%" neq "" goto :noargs3
echo.Syntax: TCUSER [/A] [/D] [/F] [/P] command username
goto :noargs4
:noargs3
echo.Syntax: TCUSER [/A] [/D] [/F] [/P] command
:noargs4
echo.
echo.  /A          Dismount all TrueCrypt volumes after operation
echo.  /D          Dismount TrueCrypt volume after operation
echo.  /F          Dismount forcibly
echo.  /P          Pause on error
echo.
echo.Administrative commands:
echo.
echo.  CONVERT     Move user profile to a TrueCrypt volume and create mount point
echo.
echo.  RESTORE     Remove mount point and copy encrypted user profile files back
echo.              to their original location
echo.
echo.  UNMAP       Remove mount point and leave user profile folder empty
echo.
echo.  MAP         Create mount point
echo.
echo.  MOUNT       Mount volume of user profile
echo.
echo.  UNMOUNT     Unmount TrueCrypt volume
echo.
echo.Logon commands:
echo.
echo.  START       Mount volume of user profile and log off
echo.  
echo.  SU          Mount volume of user profile and use "SUperior SU" to log on
if exist "%TCSU%" goto :noargs_explore
echo.              WARNING: Cannot find "SUperior SU"
echo.              (http://www.stefan-kuhr.de/supsu)

:noargs_explore
echo.
echo.  EXPLORE     Mount volume of user profile and display its contents
if exist "%TCSU%"    goto :noargs_replace
if exist "SANUR.EXE" goto :noargs_replace
echo.              WARNING: Cannot find neither "SUPerior SU" nor "SANUR.EXE"
echo.              (http://www.commandline.co.uk/sanur_unsupported/index.html)

:noargs_replace
echo.             
echo.  REPLACEWITH Mount volume of user profile, kill current explorer process and 
echo.              create a new explorer process on behalf of the encrpyted user
if exist PSKILL.EXE goto :noargs_replace2
echo.              WARNING: Cannot find "PSKILL.EXE"
echo.              (http://www.sysinternals.com/ntw2k/freeware/pskill.shtml)

:noargs_replace2
if exist "%TCSU%"    goto :noargs_epilogue
if exist "SANUR.EXE" goto :noargs_epilogue
echo.              WARNING: Cannot find neither "SUPerior SU" nor "SANUR.EXE"

:noargs_epilogue
echo.
echo.  REGISTER    Update TCGINA DLL registry entries
if "%TCSINGLEUSER%" equ "" goto :noargs_epilogue2
echo.
echo.User account: %TCUSER%
:noargs_epilogue2
echo.
echo.Note that a user account with an encrypted profile must have a password.
echo.The switches /A, /D and /F are only supported by UNMOUNT, SU and EXPLORE. 
echo. 
echo.The password request can automatically be answered if the user password is 
echo.stored at %TCDRIVE%:\%%UNAME%%.pw (where %%UNAME%% is a placeholder for the user name). 
echo.Furthermore, either "SUperior SU" or "SANUR.EXE" must be available.

if "%TCTRUECRYPTEXE%" neq "" goto :noargs_check_truecryptformat
echo.
echo.ERROR:      Cannot find TrueCrypt.exe

:noargs_check_truecryptformat
if "%TCTRUECRYPTFORMATEXE%" neq "" goto :error
echo.
echo.ERROR:      Cannot find "TrueCrypt Format.exe"
goto :error

rem +--------------------------------------------------------------------------+
rem | CONVERT                                                                  |
rem +--------------------------------------------------------------------------+

:convert
if "%TCUSER%" equ "" goto :noargs
call :search_for_truecrypt
call :search_for_truecryptformat
call :check_admin
if errorlevel 1 goto :eof
if "%TCTRUECRYPTEXE%" equ "" goto :truecryptexe_missing
if "%TCTRUECRYPTFORMATEXE%" equ "" goto :truecryptformatexe_missing
mountvol "%TCHOME%" /L >nul
if not errorlevel 1 goto :mount_point_already_exists
if not exist "%TCHOME%\ntuser.dat" goto :profile_missing
cacls "%TCHOME%\ntuser.dat" >nul
if errorlevel 1 goto :ntfs_required_for_homedir
set TCTITLEDONE=1
cls
echo.Create a new TrueCrypt volume with NTFS.
echo.Use following file name for the TrueCrypt volume:
echo.
echo.    "%TCVOL%"
echo.
echo.
"%TCTRUECRYPTFORMATEXE%"
call :check_tcvol
if errorlevel 1 goto :tcvol_missing
echo.Mount TrueCrypt volume
echo.
echo.
"%TCTRUECRYPTEXE%" %TCOPT% /v "%TCVOL%" /l %TCDRIVE%
dir %TCDRIVE%:\ | find "%TCDRIVE%:" >nul
if errorlevel 1 goto :tcdrive_missing
if exist %TCDRIVE%:\ntuser.dat goto :profile_already_exists
cacls %TCDRIVE%:\ >nul
if errorlevel 1 goto :ntfs_required_for_tcdrive
dir /b %TCDRIVE%:\*.tcuser | find ".tcuser" >nul
if not errorlevel 1 goto :tcuser_already_exists

rem Copy user profile to mounted TrueCrypt volume

xcopy "%TCHOME%\*.*" %TCDRIVE%:\ /S /E /H /K /O /F
if errorlevel 1 goto :xcopy_error
echo.>"%TCDRIVE%:\%TCUSER%.tcuser"
if not exist "%TCDRIVE%:\%TCUSER%.tcuser" goto :xcopy_error
attrib +R "%TCDRIVE%:\%TCUSER%.tcuser"
cacls %TCDRIVE%:\ /E /G "%TCUSER%":F

rem Delete original user profile files

attrib /D /S -R -H -S "%TCHOME%\*.*"
del /S /Q /F "%TCHOME%\*.*"
for /D %%i in ("%TCHOME%\*.*") do rd "%%i" /S /Q

rem Allow all users to modify the TrueCrypt volume

cacls "%TCVOL%" /E /G Users:F
cacls "%TCVOL%" /E /G Usuarios:F
cacls "%TCVOL%" /E /G Utilisateurs:F
cacls "%TCVOL%" /E /G Benutzer:F
rem <ADD CODE HERE FOR FURTHER WINDOWS LANGUAGE VERSIONS>

if "%TCCLS%" equ "1" cls
goto :map

rem +--------------------------------------------------------------------------+
rem | RESTORE                                                                  |
rem +--------------------------------------------------------------------------+

:restore
if "%TCUSER%" equ "" goto :noargs
call :check_admin
if errorlevel 1 goto :eof
call :search_for_truecrypt
if "%TCTRUECRYPTEXE%" equ "" goto :truecryptexe_missing
call :check_tcvol
if errorlevel 1 goto :tcvol_missing
dir %TCDRIVE%:\ | find "%TCDRIVE%:" >nul
if not errorlevel 1 goto :restore_tcvolume_already_mounted
if "%TCCLS%" equ "1" cls
"%TCTRUECRYPTEXE%" %TCOPT% /v "%TCVOL%" /l %TCDRIVE%
dir %TCDRIVE%:\ | find "%TCDRIVE%:" >nul
if errorlevel 1 goto :tcdrive_missing

:restore_tcvolume_already_mounted
if not exist "%TCDRIVE%:\ntuser.dat" goto :tcntuserdat_missing
if not exist "%TCDRIVE%:\%TCUSER%.tcuser" goto :tcuser_missing
if not exist "%TCHOME%\%TCUSER%.tcuser" goto :tchomeuser_missing

rem Remove mount point

mountvol "%TCHOME%" /D
if errorlevel 1 goto :mount_point_removing_error
if exist "%TCHOME%\ntuser.dat" goto :homedir_not_empty

rem Remove password files

attrib -R %TCDRIVE%:\*.pw
cacls %TCDRIVE%:\*.pw /E /G "%USERNAME%":F
del %TCDRIVE%:\*.pw

rem Copy files back to original location

xcopy %TCDRIVE%:\*.* "%TCHOME%\" /S /E /H /K /O /F /C
if errorlevel 1 goto :restore_unroll

rem Remove user identification file

attrib -R "%TCHOME%\%TCUSER%.tcuser"
cacls "%TCHOME%\%TCUSER%.tcuser" /E /G "%USERNAME%":F
del "%TCHOME%\%TCUSER%.tcuser"

rem Remove "System Volume Information" folder

cacls "%TCHOME%\System Volume Information" /E /G "%USERNAME%":F
attrib /D /S -R -H -S "%TCHOME%\System Volume Information"
del /S /Q /F "%TCHOME%\System Volume Information\*.*"
for /D %%i in ("%TCHOME%\System Volume Information\*.*") do rd "%%i" /S /Q
rd "%TCHOME%\System Volume Information"

if exist "%TCHOME%\%TCUSER%.tcuser" goto :restore_unroll
if exist "%TCHOME%\System Volume Information" goto :restore_unroll

goto :eof

:restore_unroll
if "%TCPOE%" neq "1" goto :restore_unroll2
echo.ERROR: Cannot restore profile of user "%TCUSER%"
echo.Restore will be unrolled...
pause

:restore_unroll2
cacls "%TCHOME%\System Volume Information" /E /G "%USERNAME%":F
attrib /D /S -R -H -S "%TCHOME%\*.*"
del /S /Q /F "%TCHOME%\*.*"
for /D %%i in ("%TCHOME%\*.*") do rd "%%i" /S /Q

for /f "usebackq" %%i in (`mountvol %TCDRIVE%: /L`) do set TCVOLID=%%i
mountvol "%TCHOME%" %TCVOLID%
if errorlevel 1 goto :mount_point_creation_error

goto :restore_error

rem +--------------------------------------------------------------------------+
rem | UNMAP                                                                    |
rem +--------------------------------------------------------------------------+

:unmap
if "%TCUSER%" equ "" goto :noargs
call :check_admin
if errorlevel 1 goto :eof
mountvol "%TCHOME%" /D
if errorlevel 1 goto :mount_point_removing_error
goto :eof

rem +--------------------------------------------------------------------------+
rem | MAP                                                                      |
rem +--------------------------------------------------------------------------+

:map
if "%TCUSER%" equ "" goto :noargs
call :search_for_truecrypt
call :check_admin
if errorlevel 1 goto :eof
if "%TCTRUECRYPTEXE%" equ "" goto :truecryptexe_missing
call :check_tcvol
if errorlevel 1 goto :tcvol_missing
dir %TCDRIVE%:\ | find "%TCDRIVE%:" >nul
if not errorlevel 1 goto :map_tcvolume_already_mounted
if "%TCCLS%" equ "1" cls
"%TCTRUECRYPTEXE%" %TCOPT% /v "%TCVOL%" /l %TCDRIVE%
dir %TCDRIVE%:\ | find "%TCDRIVE%:" >nul
if errorlevel 1 goto :tcdrive_missing

:map_tcvolume_already_mounted
if not exist "%TCDRIVE%:\%TCUSER%.tcuser" goto :tcuser_missing
for /f "usebackq" %%i in (`mountvol %TCDRIVE%: /L`) do set TCVOLID=%%i
mountvol "%TCHOME%" %TCVOLID%
if errorlevel 1 goto :mount_point_creation_error
goto :eof

rem +--------------------------------------------------------------------------+
rem | MOUNT                                                                    |
rem +--------------------------------------------------------------------------+

:mount
if "%TCUSER%" equ "" goto :noargs
call :search_for_truecrypt
if "%TCTRUECRYPTEXE%" equ "" goto :truecryptexe_missing
call :check_tcvol
if errorlevel 1 goto :tcvol_missing
dir %TCDRIVE%:\ | find "%TCDRIVE%:" >nul
if not errorlevel 1 goto :mount_tcvolume_already_mounted
if "%TCCLS%" equ "1" cls
"%TCTRUECRYPTEXE%" %TCOPT% /v "%TCVOL%" /l %TCDRIVE%
dir %TCDRIVE%:\ | find "%TCDRIVE%:" >nul
if errorlevel 1 goto :tcdrive_missing

:mount_tcvolume_already_mounted
if not exist "%TCDRIVE%:\ntuser.dat" goto :tcntuserdat_missing
if not exist "%TCDRIVE%:\%TCUSER%.tcuser" goto :tcuser_missing
if not exist "%TCHOME%\%TCUSER%.tcuser" goto :tchomeuser_missing

goto :eof

rem +--------------------------------------------------------------------------+
rem | UNMOUNT                                                                  |
rem +--------------------------------------------------------------------------+

:unmount
call :search_for_truecrypt
if "%TCTRUECRYPTEXE%" equ "" goto :truecryptexe_missing
set TCDOL=1
call :unmount_volumes
goto :eof

rem +--------------------------------------------------------------------------+
rem | START                                                                    |
rem +--------------------------------------------------------------------------+

:start
if "%TCUSER%" equ "" goto :noargs
call :search_for_truecrypt
if "%TCTRUECRYPTEXE%" equ "" goto :truecryptexe_missing
call :check_tcvol
if errorlevel 1 goto :tcvol_missing
dir %TCDRIVE%:\ | find "%TCDRIVE%:" >nul
if not errorlevel 1 goto :start_tcvolume_already_mounted
if "%TCCLS%" equ "1" cls
"%TCTRUECRYPTEXE%" %TCOPT% /v "%TCVOL%" /l %TCDRIVE%
dir %TCDRIVE%:\ | find "%TCDRIVE%:" >nul
if errorlevel 1 goto :tcdrive_missing

:start_tcvolume_already_mounted
if not exist "%TCDRIVE%:\ntuser.dat" goto :tcntuserdat_missing
if not exist "%TCDRIVE%:\%TCUSER%.tcuser" goto :tcuser_missing
if not exist "%TCHOME%\%TCUSER%.tcuser" goto :tchomeuser_missing

logoff

goto :eof

rem +--------------------------------------------------------------------------+
rem | SU                                                                       |
rem +--------------------------------------------------------------------------+

:su
if "%TCUSER%" equ "" goto :noargs
call :search_for_truecrypt
if "%TCTRUECRYPTEXE%" equ "" goto :truecryptexe_missing
call :check_tcvol
if errorlevel 1 goto :tcvol_missing
dir %TCDRIVE%:\ | find "%TCDRIVE%:" >nul
if not errorlevel 1 goto :su_tcvolume_already_mounted
if "%TCCLS%" equ "1" cls
"%TCTRUECRYPTEXE%" %TCOPT% /v "%TCVOL%" /l %TCDRIVE%
dir %TCDRIVE%:\ | find "%TCDRIVE%:" >nul
if errorlevel 1 goto :tcdrive_missing

:su_tcvolume_already_mounted
if not exist "%TCDRIVE%:\ntuser.dat" goto :tcntuserdat_missing
if not exist "%TCDRIVE%:\%TCUSER%.tcuser" goto :tcuser_missing
if not exist "%TCHOME%\%TCUSER%.tcuser" goto :tchomeuser_missing

if not exist "%TCSU%" goto :su_missing
if exist "%TCDRIVE%:\%TCUSER%.pw" goto :su_with_pwfile

if "%TCCLS%" equ "1" cls
"%TCSU%" "%TCUSER%" explorer.exe . "%TCUSER%"
if errorlevel 1 goto :su_error
call :unmount_volumes
goto :eof

:su_with_pwfile
if "%TCCLS%" equ "1" cls
"%TCSU%" "%TCUSER%" explorer.exe . "%TCUSER%" <"%TCDRIVE%:\%TCUSER%.pw"
if errorlevel 1 goto :su_error
call :unmount_volumes
goto :eof

rem +--------------------------------------------------------------------------+
rem | EXPLORE                                                                  |
rem +--------------------------------------------------------------------------+

:explore
if "%TCUSER%" equ "" goto :noargs
call :search_for_truecrypt
if "%TCTRUECRYPTEXE%" equ "" goto :truecryptexe_missing
call :check_tcvol
if errorlevel 1 goto :tcvol_missing
dir %TCDRIVE%:\ | find "%TCDRIVE%:" >nul
if not errorlevel 1 goto :explore_tcvolume_already_mounted
if "%TCCLS%" equ "1" cls
"%TCTRUECRYPTEXE%" %TCOPT% /v "%TCVOL%" /l %TCDRIVE%
dir %TCDRIVE%:\ | find "%TCDRIVE%:" >nul
if errorlevel 1 goto :tcdrive_missing

:explore_tcvolume_already_mounted
if not exist "%TCDRIVE%:\ntuser.dat" goto :tcntuserdat_missing
if not exist "%TCDRIVE%:\%TCUSER%.tcuser" goto :tcuser_missing
if not exist "%TCHOME%\%TCUSER%.tcuser" goto :tchomeuser_missing

if not exist "%ProgramFiles%\Internet Explorer\iexplore.exe" goto :iexplorer_missing
if not exist "%TCDRIVE%:\%TCUSER%.pw" goto :explore_without_pwfile
if not exist sanur.exe goto :explore_without_sanur

if "%TCCLS%" equ "1" cls
runas /user:"%TCUSER%" "%ProgramFiles%\Internet Explorer\iexplore.exe %TCDRIVE%:" | sanur /i "%TCDRIVE%:\%TCUSER%.pw"
goto :eof

:explore_without_sanur
if not exist "%TCSU%" goto :explore_without_pwfile
if "%TCCLS%" equ "1" cls
"%TCSU%" "%TCUSER%" "%ProgramFiles%\Internet Explorer\iexplore.exe %TCDRIVE%:" <"%TCDRIVE%:\%TCUSER%.pw"
call :unmount_volumes
goto :eof

:explore_without_pwfile
if "%TCCLS%" equ "1" cls
runas /user:"%TCUSER%" "%ProgramFiles%\Internet Explorer\iexplore.exe %TCDRIVE%:"
goto :eof

rem +--------------------------------------------------------------------------+
rem | REPLACEWITH                                                              |
rem +--------------------------------------------------------------------------+

:replacewith
if "%TCUSER%" equ "" goto :noargs
call :search_for_truecrypt
if not exist PSKILL.EXE goto :pskill_missing
if "%TCTRUECRYPTEXE%" equ "" goto :truecryptexe_missing
call :check_tcvol 
if errorlevel 1 goto :tcvol_missing
dir %TCDRIVE%:\ | find "%TCDRIVE%:" >nul
if not errorlevel 1 goto :replacewith_tcvolume_already_mounted
if "%TCCLS%" equ "1" cls
"%TCTRUECRYPTEXE%" %TCOPT% /v "%TCVOL%" /l %TCDRIVE%
dir %TCDRIVE%:\ | find "%TCDRIVE%:" >nul
if errorlevel 1 goto :tcdrive_missing

:replacewith_tcvolume_already_mounted
if not exist "%TCDRIVE%:\ntuser.dat" goto :tcntuserdat_missing
if not exist "%TCDRIVE%:\%TCUSER%.tcuser" goto :tcuser_missing
if not exist "%TCHOME%\%TCUSER%.tcuser" goto :tchomeuser_missing

if not exist "%ProgramFiles%\Internet Explorer\iexplore.exe" goto :iexplorer_missing
if not exist "%TCDRIVE%:\%TCUSER%.pw" goto :replacewith_without_pwfile
if not exist sanur.exe goto :replacewith_without_sanur

pskill explorer.exe
runas /user:"%TCUSER%" explorer.exe | sanur /i "%TCDRIVE%:\%TCUSER%.pw"
goto :eof

:replacewith_without_sanur
if not exist "%TCSU%" goto :replacewith_without_pwfile
pskill explorer.exe
"%TCSU%" "%TCUSER%" explorer.exe <"%TCDRIVE%:\%TCUSER%.pw" -w
goto :eof

:replacewith_without_pwfile
pskill explorer.exe
runas /user:"%TCUSER%" explorer.exe
goto :eof

rem +--------------------------------------------------------------------------+
rem | REGISTER                                                                 |
rem +--------------------------------------------------------------------------+

:register
if "%TCUSER%" equ "" goto :noargs
call :check_admin
if errorlevel 1 goto :eof
if not exist reg.exe goto :regexe_missing
reg SET REG_SZ "HKLM\SOFTWARE\TCGINA DLL\{D47DF546-F16B-490b-A6D8-523603A5594D}\%TCUSER%" TCDrive "%TCDRIVE%"
reg SET REG_SZ "HKLM\SOFTWARE\TCGINA DLL\{D47DF546-F16B-490b-A6D8-523603A5594D}\%TCUSER%" TCFileName "%TCVOL%"
goto :eof

rem +--------------------------------------------------------------------------+
rem | Error messages                                                           |
rem +--------------------------------------------------------------------------+

:noextensions
call :print_title
echo.ERROR: Cannot execute "SETLOCAL ENABLEEXTENSIONS"
goto :error

:das_missing
call :print_title
echo.ERROR: Cannot find "Documents and Settings" folder
goto :error

:regexe_missing
call :print_title
echo.ERROR: Cannot find "reg.exe"
goto :error

:username_missing
call :print_title
echo.ERROR: The TrueCrypt volume name can only be changed if TCUSER is specified!
goto :error

:truecryptexe_missing
call :print_title
echo.ERROR: Cannot find TrueCrypt.exe
goto :error

:truecryptformatexe_missing
call :print_title
echo.ERROR: Cannot find "TrueCrypt Format.exe"
goto :error

:tcvol_missing
call :print_title
echo.ERROR: Cannot find TrueCrypt file "%TCVOL%"
goto :error

:tcdrive_missing
call :print_title
echo.ERROR: Cannot find TrueCrypt volume %TCDRIVE%:
goto :error

:profile_missing
call :print_title
echo.ERROR: Cannot find profile of user "%TCHOME%"
goto :error

:iexplorer_missing
call :print_title
echo.ERROR: Cannot find "%ProgramFiles%\Internet Explorer\iexplore.exe"
goto :error

:invalid_username
call :print_title
echo.ERROR: TCUSER cannot be used for current user ("%USERNAME%")
goto :error

:mount_point_already_exists
call :print_title
echo.ERROR: Mount point already exists
goto :error

:ntfs_required_for_homedir
call :print_title
echo.ERROR: "%TCHOME%" is not located on a NTFS volume
goto :error

:ntfs_required_for_tcdrive
call :print_title
echo.ERROR: %TCDRIVE%: is not a NTFS volume
goto :error

:profile_already_exists
call :print_title
echo.ERROR: %TCDRIVE%: already contains a user profile
goto :error

:tcuser_already_exists
call :print_title
echo.ERROR: Profiles of following users found on TrueCrypt volume %TCDRIVE%:
dir /b %TCDRIVE%:\*.tcuser
goto :error

:tcuser_missing
call :print_title
echo.ERROR: Files for user "%TCUSER%" not found on %TCDRIVE%: ("%TCUSER%.tcuser" not found)
goto :error

:tchomeuser_missing
call :print_title
echo.ERROR: Files for user "%TCUSER%" not found at "%TCHOME%" ("%TCUSER%.tcuser" not found)
goto :error

:xcopy_error
call :print_title
echo.ERROR: Cannot copy user profile to mounted TrueCrypt volume
goto :error

:mount_point_creation_error
call :print_title
echo.ERROR: Cannot create mount point
goto :error

:mount_point_removing_error
call :print_title
echo.ERROR: Cannot remove mount point
goto :error

:tcntuserdat_missing
call :print_title
echo.ERROR: Encrypted registry hive not found on %TCDRIVE%: (%TCDRIVE%:\ntuser.dat)
goto :error

:homedir_not_empty
call :print_title
echo.ERROR: "%TCHOME%" is not empty
goto :error

:restore_error
call :print_title
echo.ERROR: Cannot restore profile of user "%TCUSER%"
goto :error

:su_missing
call :print_title
echo.ERROR: Cannot find SU.EXE (http://www.stefan-kuhr.de/supsu)
goto :error

:pskill_missing
call :print_title
echo.ERROR: Cannot find PSKILL.EXE
echo.       (http://www.sysinternals.com/ntw2k/freeware/pskill.shtml)

:su_error
"%TCTRUECRYPTEXE%" /d /q /l %TCDRIVE%
call :print_title
echo.ERROR: Cannot logon "%TCUSER%"
goto :error

rem +--------------------------------------------------------------------------+
rem | Subroutines                                                              |
rem +--------------------------------------------------------------------------+

:set_errorlevel_0
setlocal ENABLEEXTENSIONS
endlocal
goto :eof

:set_errorlevel_1
verify OTHER 2 >nul
goto :eof

rem ----------------------------------------------------------------------------

:extract_path
set STR=%~dp1
if "%STR%" equ "" goto :eof
set STR=%STR:~0,-1%
goto :eof

rem ----------------------------------------------------------------------------

:print_title
if "%TCTITLEDONE%" equ "1" goto :eof
echo.TCUSER V%TCUSERVER%
echo.
set TCTITLEDONE=1
goto :eof

rem ----------------------------------------------------------------------------

:error
if not "%TCPOE%" neq "1" pause
goto :eof

rem ----------------------------------------------------------------------------

:check_admin
call :check_os
if errorlevel 1 goto :check_admin_wrong_os
sc query truecrypt | more +3 | find "RUNNING" >nul
if errorlevel 1 goto :check_admin_notruecrypt
sc start truecrypt | find "FAILED 1056" >nul
if errorlevel 1 goto :check_admin_noadmin
call :set_errorlevel_0
goto :eof

:check_admin_wrong_os
call :print_title
echo.ERROR: Windows XP or Windows 2003 required
echo.(This message can also appear if you use an alternative command line
echo.processor like 4NT. In this case execute "cmd" before executing TCUSER.CMD)
goto :eof

:check_admin_notruecrypt
call :print_title
echo.ERROR: TrueCrypt not found
goto :eof

:check_admin_noadmin
call :print_title
echo.ERROR: Administrator privileges required
goto :eof

rem ----------------------------------------------------------------------------

:check_os
ver | find "Microsoft Windows" >nul
if errorlevel 1 goto :eof
ver | find "Version 5.1." >nul
if not errorlevel 1 goto :eof
ver | find "Version 5.2." >nul
goto :eof

rem ----------------------------------------------------------------------------

:check_tcvol
call :set_errorlevel_0
if "%TCVOL:~0,16%" equ "\Device\Harddisk" goto :eof
dir "%TCVOL%" >NUL
goto :eof

rem ----------------------------------------------------------------------------

:search_for_truecrypt
set TCTRUECRYPTEXE=
if exist TrueCrypt.exe goto :search_for_truecrypt_cmdhome
if exist "%ProgramFiles86%\TrueCrypt\TrueCrypt.exe" goto :search_for_truecrypt_programfiles
goto :eof

:search_for_truecryptformat
set TCTRUECRYPTFORMATEXE=
if exist "TrueCrypt Format.exe" goto :search_for_truecryptformat_cmdhome
if exist "%ProgramFiles86%\TrueCrypt\TrueCrypt Format.exe" goto :search_for_truecryptformat_programfiles
goto :eof

:search_for_truecrypt_cmdhome
set TCTRUECRYPTEXE=TrueCrypt.exe
goto :eof

:search_for_truecrypt_programfiles
set TCTRUECRYPTEXE=%ProgramFiles86%\TrueCrypt\TrueCrypt.exe
goto :eof

:search_for_truecryptformat_cmdhome
set TCTRUECRYPTFORMATEXE=TrueCrypt Format.exe
goto :eof

:search_for_truecryptformat_programfiles
set TCTRUECRYPTFORMATEXE=%ProgramFiles86%\TrueCrypt\TrueCrypt Format.exe
goto :eof

rem ----------------------------------------------------------------------------

:update_dol
if "%TCFOL%" equ "1" set TCDOL=1
if "%TCAOL%" equ "1" set TCDOL=1
goto:eof

:unmount_volumes
call :update_dol
if "%TCDOL%" neq "1" goto :eof
set STR=
if not "%TCAOL%" equ "1" set STR=%TCDRIVE%
if "%TCFOL%" equ "1" set STR=%STR% /f
"%TCTRUECRYPTEXE%" /q /d %STR%
goto :eof
